
local myname, ns = ...
local _G = getfenv(0)

--=================================================
-- Upvalued functions and variables
--=================================================

local print		= _G.print
local select	= _G.select


--=================================================
-- Utility functions and constants
--=================================================

local myfullname = _G.GetAddOnMetadata(myname, "Title")
function ns.Print(...) print("|cFF33FF99".. myfullname.. "|r:", ...) end
local Print = ns.Print

function ns.err(msg, ...) _G.geterrorhandler()(msg:format(_G.tostringall(...)) .. " - " .. _G.time()) end
local err = ns.err

local IS_MOP = select(4, _G.GetBuildInfo()) >= 50000
local ACTIVE_QUEST_PARAMS = IS_MOP and 5 or 4

--=================================================
-- Even Dispatcher
--=================================================

do -- Closure

	local f = _G.CreateFrame("frame")
	f:SetScript("OnEvent", function(self, event, ...) if ns[event] then return ns[event](ns, event, ...) else ns:OtherEvent(event, ...) end end)

	function ns:RegisterEvent(...) for i=1,select("#", ...) do f:RegisterEvent((select(i, ...))) end end
	function ns:UnregisterEvent(...) for i=1,select("#", ...) do f:UnregisterEvent((select(i, ...))) end end

end -- Closure

--=================================================
-- Init addon
--=================================================

ns:RegisterEvent("ADDON_LOADED")
function ns:ADDON_LOADED(event, addon)
	if addon ~= myname then return end

	-- Events
	self:RegisterEvent(
		"GOSSIP_SHOW",
		"QUEST_COMPLETE",
		"QUEST_DETAIL",
		"QUEST_GREETING",
		"QUEST_PROGRESS"
	)

	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil

	if _G.IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent("PLAYER_LOGIN") end
end


function ns:PLAYER_LOGIN()
	self:UnregisterEvent("PLAYER_LOGIN")
	self.PLAYER_LOGIN = nil
end

--=================================================
-- Core functionality
--=================================================

-- Hook the quest Decline button so that if we go back to the gossip frame, it doesn't
-- trigger the auto-accept immediately
local just_declined = nil
do -- Closure

	local DELAY_TIME = 0.5
	local delay
	local f = _G.CreateFrame("Frame")
	f:Hide()
	f:SetScript("OnUpdate", function(self, elapsed)
		delay = delay - elapsed
		if delay < 0 then
			just_declined = nil
			f:Hide()
		end
	end)

	_G.QuestFrameDeclineButton:HookScript("OnClick", function()
		just_declined = true
		delay = DELAY_TIME
		f:Show()
	end)

end -- Closure


-- Options to select if available
local good_options = {
	trainer = true,
	vendor = true,
}

function ns:OtherEvent(event, ...)
	-- If the shift key is pressed or if we just declined a quest, do nothing
	if just_declined or _G.IsShiftKeyDown() then return end

	if event == "GOSSIP_SHOW" then

		-- Turn in the firt completed quest of the list
		for i=1,_G.GetNumGossipActiveQuests() do
			local is_completed = select((i-1)*ACTIVE_QUEST_PARAMS + 4, _G.GetGossipActiveQuests())
			if is_completed then
				_G.SelectGossipActiveQuest(i)
				return
			end
		end

		-- If there is only one selectable quest
		if _G.GetNumGossipAvailableQuests() == 1 then
			_G.SelectGossipAvailableQuest(1)
			return
		end

		-- Find the other interesting stuff and oppen the corresponding option
		for i=1,_G.GetNumGossipOptions() do
			local option = select(i*2, _G.GetGossipOptions()) -- (i-1)*2 + 2
			if good_options[option] then
				_G.SelectGossipOption(i)
				return
			end
		end

	elseif event == "QUEST_DETAIL" then

		if _G.QuestIsDaily() then
			-- Auto Quest are automaticaly accepted as soon as you see the quest text
			-- This code was lifted from QuestFrame.lua to prevent calling AcceptQuest()
			-- for an already accepted quest.
			if ( _G.QuestFrame.autoQuest ) then
				_G.HideUIPanel(_G.QuestFrame)
			else
				_G.AcceptQuest()
			end
		end

	elseif event == "QUEST_PROGRESS" then

		-- Autoselect the Continue button when the quest is completed
		if _G.IsQuestCompletable() then
			_G.CompleteQuest()
		end

	elseif event == "QUEST_COMPLETE" then

		-- Choose the reward automaticaly if there is less then two choices available
		if _G.GetNumQuestChoices() <= 1 then
				_G.GetQuestReward(0)
		end

	end
end
